package com.mswipetech.wisepos.demo.sdk.util;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.text.Html;

import com.mswipetech.wisepad.sdk.data.ReceiptData;
import com.mswipetech.wisepos.demo.sdk.R;
import com.mswipetech.wisepos.demo.sdk.data.ApplicationData;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;

import BpPrinter.mylibrary.BpPrinter;

public class ReceiptUtilityBlu {

	private  DecimalFormat df = new DecimalFormat(".00");


	private Context context = null;
	private ReceiptData mReceiptDataModel = null;

	private boolean isPrintSignatureRequired = false;
	private Bitmap croppedBitmap = null;

	private final int mswipeTargetWidth = 380;
	private boolean isCustomerCopy = false;
	private boolean isDuplicatePrint = false;

	private String mTotalAmount = "";
	private String mBatchNo = "";
	private String mStoreName = "";
	private String mOpendate = "";
	private String mClosedate = "";
	private boolean isNbfcTrx = false;


	private final  byte FONT_SMALL 	= (byte) 0x01;
	private final  byte FONT_MEDIUM = (byte) 0x02;
	private final  byte FONT_LARGE 	= (byte) 0x00;

	private final  byte FONT_BOLD 	= (byte) 0x08;

	private final  byte ALIGN_LEFT 		= (byte) 0x00;
	private final  byte ALIGN_CENTER 	= (byte) 0x01;
	private final  byte ALIGN_RIGHT		= (byte) 0x02;

	public enum TYPE{
		CARD, EMI, CASH
	}

	BpPrinter mBluPrinter = null;

	public ReceiptUtilityBlu(Context context, BpPrinter mBluPrinter){

		this.context = context;
		this.mBluPrinter = mBluPrinter;

		try {
			mBluPrinter.Initialize_Printer();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public boolean printReceipt(ReceiptData aReceiptDataModel, Bitmap bitmap, boolean isPrintSignatureRequired, TYPE type){

		if (ApplicationData.IS_DEBUGGING_ON)
			Logs.v(ApplicationData.packName, " type " + type, true, true);

		this.mReceiptDataModel = aReceiptDataModel;
		this.isPrintSignatureRequired = isPrintSignatureRequired;

		if (bitmap != null) {

			croppedBitmap = Bitmap.createBitmap(bitmap, 0, 0, mswipeTargetWidth, bitmap.getHeight());

		}else{

			this.isPrintSignatureRequired = false;
		}

		if (type == TYPE.CARD) {

			return genCardSaleReceipt();

		}else if(type == TYPE.EMI){

			return genEmiSaleReceipt();

		}else if(type == TYPE.CASH){

			return genCashSaleReceipt();
		}

		return false;
	}

	private boolean genCardSaleReceipt()
	{

		if (ApplicationData.IS_DEBUGGING_ON)
			Logs.v(ApplicationData.packName, " genCradSaleReceipt " , true, true);

		try {


			String strTipLbl = "TIP AMOUNT:    ";
			if(mReceiptDataModel.isConvenceFeeEnable.equalsIgnoreCase("true")){
				strTipLbl = "CONVENIENCE FEE:";
			}

			if (ApplicationData.IS_DEBUGGING_ON)
				Logs.v(ApplicationData.packName, " isConvenceFeeEnable " + mReceiptDataModel.isConvenceFeeEnable + strTipLbl, true, true);



			double tipamount = 0.00;
			if (mReceiptDataModel.tipAmount.length() > 0 ) {
				tipamount = Double.parseDouble(mReceiptDataModel.tipAmount);
			}

			mBluPrinter.POS_LineSpacing((byte) 2);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			Bitmap icon = getHeaderLogo();

			if(icon != null)
			{
				int targetWidth = 150;
				int targetHeight = icon.getHeight();

				Bitmap scaledBitmap = Bitmap.createScaledBitmap(icon, targetWidth, targetHeight, false);
				mBluPrinter.printImage(scaledBitmap, 0);
			}


			mBluPrinter.setLineFeed(1);

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.print(mReceiptDataModel.merchantName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			String splitStr = "<br />";

			if (mReceiptDataModel.merchantAdd.contains("\n")) {
				splitStr = "\n";
			}

			for (String add : mReceiptDataModel.merchantAdd.split(splitStr)) {

				if (ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, "address  " + add, true, true);

				mBluPrinter.print(add + "\n");

			}


			String tempDate = removeampm(mReceiptDataModel.dateTime);
			String dateTime = Constants.getDateWithFormate(tempDate, "dd-MMMM-yyyy HH:mm", "dd MMM yyyy hh:mm a");

			if (ApplicationData.IS_DEBUGGING_ON)
				Logs.v(ApplicationData.packName, " dateTime "+dateTime , true, true);

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			String disdateTime = "Date/Time: " + " " + dateTime.toUpperCase();
			printMediumFontLine(disdateTime + "\n");

			String mMidTid = "";
			if(mReceiptDataModel.mId != null && mReceiptDataModel.mId.length() > 0) {
				mMidTid = "MID:" + mReceiptDataModel.mId;
			}

			if(mReceiptDataModel.tId != null && mReceiptDataModel.tId.length() > 0) {
				mMidTid = mMidTid + " " + "TID:" + mReceiptDataModel.tId;
			}

			if(mMidTid.length() > 0) {
				mBluPrinter.print(mMidTid+ "\n");
			}

			String mBatchInvoide = "";
			if(mReceiptDataModel.trxBatchNo != null && mReceiptDataModel.trxBatchNo.length() > 0) {
				mBatchInvoide = "Batch no:" + mReceiptDataModel.trxBatchNo + " ";
			}
			if(mReceiptDataModel.invoiceNo != null && mReceiptDataModel.invoiceNo.length() > 0) {
				mBatchInvoide = mBatchInvoide + "Inv. no:" + mReceiptDataModel.invoiceNo.replace("\n", "");
			}

			if(mBatchInvoide.length() > 0) {
				mBluPrinter.print(mBatchInvoide + "\n");
			}

			if(mReceiptDataModel.refNo != null && mReceiptDataModel.refNo.length() > 0) {
				mBluPrinter.print("Ref no.:" + " " + mReceiptDataModel.refNo + "\n");
			}

			if (mReceiptDataModel.cashAmount.length() > 0 && mReceiptDataModel.tipAmount.length() <= 0 && mReceiptDataModel.baseAmount.length()<= 0) {

			}
			else {

				mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);
				//mBluPrinter.POS_Set_Char_Mode(FONT_BOLD);

				mBluPrinter.print(mReceiptDataModel.saleType.toUpperCase() + "\n");

			}

			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);
			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printMediumFontLine("Card no.:" + mReceiptDataModel.cardNo+" "+mReceiptDataModel.trxType+ "\n");


			mBluPrinter.print("Card type: " + mReceiptDataModel.cardType + "    Exp dt: " + "xx/xx" + "\n");

			if (mReceiptDataModel.cashAmount.length() > 0 && mReceiptDataModel.baseAmount.length() > 0) {

				mBluPrinter.print(("SALE AMT:   " + mReceiptDataModel.currency + " " +mReceiptDataModel.baseAmount) + "\n");

				mBluPrinter.print(("CASH PAID:    " + mReceiptDataModel.currency + " " +mReceiptDataModel.cashAmount) + "\n");


				if (mReceiptDataModel.tipAmount.length() > 0 && tipamount >0) {

					mBluPrinter.print(strTipLbl + mReceiptDataModel.currency + " " +mReceiptDataModel.tipAmount + "\n");
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print(("TOTAL AMOUNT:   " + mReceiptDataModel.currency + " " +mReceiptDataModel.totalAmount) + "\n");

			}
			else if (mReceiptDataModel.cashAmount.length() > 0 && mReceiptDataModel.baseAmount.length()<= 0) {

				if (mReceiptDataModel.tipAmount.length() > 0 && tipamount >0) {

					mBluPrinter.print(("SALE AMT:   " + mReceiptDataModel.currency + " 0.00") + "\n");

					mBluPrinter.print(("CASH PAID:    " + mReceiptDataModel.currency + " " +mReceiptDataModel.cashAmount) + "\n");

					mBluPrinter.print(strTipLbl + mReceiptDataModel.currency + " " +mReceiptDataModel.tipAmount + "\n");

					mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

					printLine();

					mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

					mBluPrinter.print(("TOTAL AMOUNT:   " + mReceiptDataModel.currency + " " +mReceiptDataModel.totalAmount) + "\n");

				}else{
					mBluPrinter.print(("CASH PAID:    " + mReceiptDataModel.currency + " " +mReceiptDataModel.cashAmount) + "\n");

				}

			}else{

				if (mReceiptDataModel.tipAmount.length() > 0 && tipamount >0) {

					mBluPrinter.print(("BASE AMOUNT:    " + mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount) + "\n");

					mBluPrinter.print((strTipLbl + mReceiptDataModel.currency + " " +mReceiptDataModel.tipAmount) + "\n");

					mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

					printLine();

					mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

					mBluPrinter.print(("TOTAL AMOUNT:   " + mReceiptDataModel.currency + " " +mReceiptDataModel.totalAmount) + "\n");

				}else{

					mBluPrinter.print(("TOTAL AMOUNT:   " + mReceiptDataModel.currency + " " +mReceiptDataModel.totalAmount) + "\n");

				}
			}

			mBluPrinter.print("Appr cd:" + mReceiptDataModel.authCode + " RRN:" + mReceiptDataModel.rrNo + "\n");

			if (mReceiptDataModel.trxType.equalsIgnoreCase("Chip")) {

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("TC:" + mReceiptDataModel.certif + "\n");

				mBluPrinter.print("App id:" + mReceiptDataModel.appId + "\n");

				mBluPrinter.print("App name:" + mReceiptDataModel.appName + "\n");

				mBluPrinter.print("TVR:" + mReceiptDataModel.tvr.trim() + " TSI:" + mReceiptDataModel.tsi.trim() + "\n");

			}

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			if (mReceiptDataModel.isPinVarifed.equalsIgnoreCase("true")) {

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.print("Pin Verified Ok" + "\n");

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);


			}else{

				if (isPrintSignatureRequired) {

					mBluPrinter.printImage(croppedBitmap, 0);

				}else{

					mBluPrinter.print("               " + "\n");
					mBluPrinter.print("               " + "\n");

					mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

					mBluPrinter.print("  signature" + "\n");

					mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

					mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

					printLine();

					mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
				}
			}

			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			mBluPrinter.print(mReceiptDataModel.cardHolderName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("I AGREE TO PAY THE ABOVE TOTAL AMOUNT" + "\n");

			mBluPrinter.print("ACCORDING TO THE CARD ISSUER AGREEMENT" + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			if(isCustomerCopy){

				if (ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, " CUSTOMER COPY ", true, true);

				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  CUSTOMER COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** CUSTOMER COPY ***" + "\n");
				}
			}
			else {

				if (ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, " MERCHANT COPY ", true, true);

				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  MERCHANT COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** MERCHANT COPY ***" + "\n");
				}
			}

			mBluPrinter.print("Version No : " + mReceiptDataModel.appVersion + "\n");


			String bankName = getSettlementBank(mReceiptDataModel.bankName);

			if(bankName != null)
			{
				mBluPrinter.print("Settlement Bank :" + bankName + "\n");
			}

			mBluPrinter.setLineFeed(6);

			try {
				Thread.sleep(4000);
			}  catch(Exception e) {
				e.printStackTrace();
			}

			return true;

		} catch(Exception e) {
			e.printStackTrace();
		}

		return false;
	}


	private  boolean genEmiSaleReceipt()
	{

		if (ApplicationData.IS_DEBUGGING_ON)
			Logs.v(ApplicationData.packName, " genEmiSaleReceipt " , true, true);

		if (isNbfcTrx){
			return generateNBFCEMiSaleReceipt();
		}

		try {

			StringBuilder cardNumMask = new StringBuilder();

			if ((mReceiptDataModel.firstDigitsOfCard).length() >= 6) {

				cardNumMask.append((mReceiptDataModel.firstDigitsOfCard).substring(0, 4) + " " + (mReceiptDataModel.firstDigitsOfCard).substring(4, 6));
			}

			if (mReceiptDataModel.cardNo.length() >= 8) {

				cardNumMask.append((mReceiptDataModel.cardNo).substring(8, mReceiptDataModel.cardNo.length()));
			}

			mReceiptDataModel.cardNo = cardNumMask.toString();

			if (mReceiptDataModel.emiPerMonthAmount.length() <= 0) {
				mReceiptDataModel.emiPerMonthAmount = "0.00";
			}

			if (mReceiptDataModel.total_Pay_Amount.length() <= 0) {
				mReceiptDataModel.total_Pay_Amount = "0.00";
			}

			double emiPerMonthAmountDouble = Double.parseDouble(mReceiptDataModel.emiPerMonthAmount.toString());
			double totalPayAmountDouble = Double.parseDouble(mReceiptDataModel.total_Pay_Amount.toString());

			mBluPrinter.POS_LineSpacing((byte) 2);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			Bitmap icon = getHeaderLogo();

			if(icon != null)
			{
				int targetWidth = 150;
				int targetHeight = icon.getHeight();

				Bitmap scaledBitmap = Bitmap.createScaledBitmap(icon, targetWidth, targetHeight, false);

				mBluPrinter.printImage(scaledBitmap, 0);

			}

			mBluPrinter.setLineFeed(1);
			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			mBluPrinter.print(mReceiptDataModel.merchantName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			String splitStr = "<br />";

			if (mReceiptDataModel.merchantAdd.contains("\n")) {
				splitStr = "\n";
			}

			for (String add : mReceiptDataModel.merchantAdd.split(splitStr)) {

				if (ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, "address  " + add, true, true);

				mBluPrinter.print(add + "\n");

			}


			String tempDate = removeampm(mReceiptDataModel.dateTime);
			String dateTime = Constants.getDateWithFormate(tempDate, "dd-MMMM-yyyy HH:mm", "dd MMM yyyy hh:mm a");


			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			printLine();
			printMediumFontLine("Date/Time: " + dateTime + "\n");

			mBluPrinter.print("MID: " + mReceiptDataModel.mId + " TID: " + mReceiptDataModel.tId + "\n");

			String mBatchInvoide = "";
			if(mReceiptDataModel.trxBatchNo != null && mReceiptDataModel.trxBatchNo.length() > 0) {
				mBatchInvoide = "Batch no:" + mReceiptDataModel.trxBatchNo + " ";
			}
			if(mReceiptDataModel.invoiceNo != null && mReceiptDataModel.invoiceNo.length() > 0) {
				mBatchInvoide = mBatchInvoide + "Inv. no:" + mReceiptDataModel.invoiceNo.replace("\n", "");
			}

			mBluPrinter.print(mBatchInvoide + "\n");

			mBluPrinter.print("Bill No.: " + mReceiptDataModel.billNo + "\n");

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("EMI" + "\n");

			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			printMediumFontLine("Card no.:" + mReceiptDataModel.cardNo + " " + mReceiptDataModel.trxType  + "\n");

			mBluPrinter.print("Exp dt:" + "xx/xx" + "\n");

			mBluPrinter.print("Card type:" + mReceiptDataModel.cardType + "\n");

			mBluPrinter.print("Appr.Code: " + mReceiptDataModel.authCode + " RRN: " + mReceiptDataModel.rrNo + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			if (mReceiptDataModel.trxType.equalsIgnoreCase("Chip")) {

				mBluPrinter.print("TC: " + mReceiptDataModel.certif + "\n");

				mBluPrinter.print("APP. ID: " + mReceiptDataModel.appId + "\n");

				mBluPrinter.print("APP. Name: " + mReceiptDataModel.appName + "\n");

				mBluPrinter.print("TVR: " + mReceiptDataModel.tvr.trim() + " TSI: " + mReceiptDataModel.tsi.trim() + "\n");

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			}

			if (mReceiptDataModel.MfrName != null && mReceiptDataModel.MfrName.length() > 0)
			{

				mBluPrinter.print("Product Details" + "\n");

				mBluPrinter.print("Brand Name: " + mReceiptDataModel.MfrName + "\n");

				mBluPrinter.print("Prod Category: " + mReceiptDataModel.ProdCategory + "\n");

				mBluPrinter.print("Prod Name: " + mReceiptDataModel.ProdDesc + "\n");

				mBluPrinter.print("Prod Serial: " + mReceiptDataModel.ProdSerial + "\n");

				mBluPrinter.print("Offer Name: " + mReceiptDataModel.OfferTitle + "\n");

				mBluPrinter.print("Mobile: " + mReceiptDataModel.Mobile + "\n");

				double instDiscPer = 0;
				double baseAmount = 0;

				if(mReceiptDataModel.instDiscPer != null && mReceiptDataModel.instDiscPer.trim().length() > 0)
				{
					instDiscPer = Double.parseDouble(Constants.removeChar(mReceiptDataModel.instDiscPer,','));

					if(instDiscPer > 0)
					{
						String strInstDiscPer = Constants.getAmountWithComma(String.format("%.2f", instDiscPer));

						baseAmount = Double.parseDouble(Constants.removeChar(mReceiptDataModel.baseAmount, ','));
						String strbaseAmount = Constants.getAmountWithComma(String.format("%.2f", (baseAmount + instDiscPer)));

						mBluPrinter.print("Prod Amt: " + mReceiptDataModel.currency + " " + strbaseAmount + "\n");

						mBluPrinter.print("Inst Discount: " + mReceiptDataModel.currency + " " + strInstDiscPer + "\n");

					}
				}

				String loanAmt = Constants.getAmountWithComma(String.format("%.2f", (Double.parseDouble(mReceiptDataModel.loanAmount))));
				String merchPaybackAmt = Constants.getAmountWithComma(String.format("%.2f", Double.parseDouble(mReceiptDataModel.discount)));
				String emiPerMonthAmt = Constants.getAmountWithComma(String.format("%.2f", Double.parseDouble(mReceiptDataModel.emiPerMonthAmount)));
				String totalAmt = Constants.getAmountWithComma(String.format("%.2f", Double.parseDouble(mReceiptDataModel.total_Pay_Amount)));

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("EMI Txn ID: " + mReceiptDataModel.billNo + "\n");

				mBluPrinter.print("Tenure: " + mReceiptDataModel.noOfEmi + " Months" + "\n");

				mBluPrinter.print("Card Issuer: " + mReceiptDataModel.cardIssuer + "\n");

				mBluPrinter.print("Trx Amt: " + mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount + "\n");

				mBluPrinter.print("Merch Payback: " + mReceiptDataModel.emiPer + "%" + "\n");

				mBluPrinter.print("Merch Payback Amt: " + merchPaybackAmt + "\n");

				mBluPrinter.print("Loan amount: " + loanAmt + "\n");

				mBluPrinter.print("Easy Emi Fin Charge: " + mReceiptDataModel.emiRate  + "%" + "\n");

				mBluPrinter.print("EMI Amt: " + mReceiptDataModel.currency + " " + emiPerMonthAmt + "\n");

				mBluPrinter.print("Total Amount:  "+ (mReceiptDataModel.currency + " " + totalAmt) + "\n");

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("CUSTOMER CONSENT FOR EMI" + "\n");

				if(ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, " TermsConditionse " + mReceiptDataModel.TermsConditionse , true, true);
				if(ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, " CBTermsConditionse " + mReceiptDataModel.CBTermsConditionse , true, true);

				if(mReceiptDataModel.OfferTitle.equalsIgnoreCase("cashback"))
				{
					if(mReceiptDataModel.CBTermsConditionse != null && mReceiptDataModel.CBTermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.CBTermsConditionse + "\n");

					}
				}
				else if(mReceiptDataModel.OfferTitle.equalsIgnoreCase("zero emi"))
				{
					if(mReceiptDataModel.TermsConditionse != null && mReceiptDataModel.TermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.TermsConditionse + "\n");

					}
				}
				else
				{
					if(mReceiptDataModel.TermsConditionse != null && mReceiptDataModel.TermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.TermsConditionse + "\n");

					}

					if(mReceiptDataModel.CBTermsConditionse != null && mReceiptDataModel.CBTermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.CBTermsConditionse + "\n");

					}
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			}
			else {

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("EMI Txn ID: " + mReceiptDataModel.billNo + "\n");

				mBluPrinter.print("Tenure: " + mReceiptDataModel.noOfEmi + " Months" + "\n");

				mBluPrinter.print("Card Issuer: " + mReceiptDataModel.cardIssuer + "\n");

				mBluPrinter.print("Base Amt: " +  mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount + "\n");

				mBluPrinter.print("Applicable Intr. Rate (P.A.) : " + mReceiptDataModel.interestRate + "%" + "\n");

				mBluPrinter.print("EMI Amt: " + mReceiptDataModel.currency + " " + df.format(emiPerMonthAmountDouble).toString() + "\n");

				mBluPrinter.print("Total Amt Payable: " +  mReceiptDataModel.currency + " " + df.format(totalPayAmountDouble).toString() + "\n");

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("CUSTOMER CONSENT FOR EMI" + "\n");

				if(mReceiptDataModel.TermsConditionse != null && mReceiptDataModel.TermsConditionse.length() >0)
				{
					String splitStrr = "<br />";

					if (mReceiptDataModel.TermsConditionse.contains("\n")) {
						splitStrr = "\n";
					}

					for (String add : mReceiptDataModel.TermsConditionse.split(splitStrr)) {

						if (ApplicationData.IS_DEBUGGING_ON)
							Logs.v(ApplicationData.packName, "address  " + add, true, true);


						mBluPrinter.print(add + "\n");
					}
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("Base Amt: " + mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount + "\n");

				printLargeFontLine("Total Amt Payable (Incl. Interest): " + mReceiptDataModel.currency + " " + df.format(totalPayAmountDouble) + "\n");

			}


			if (mReceiptDataModel.isPinVarifed.equalsIgnoreCase("true")) {

				mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

				mBluPrinter.print("Pin Verified Ok" + "\n");

				mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			}else{

				if (isPrintSignatureRequired) {

					mBluPrinter.printImage(croppedBitmap, 0);

				}else{

					mBluPrinter.print("               " + "\n");
					mBluPrinter.print("               " + "\n");

					mBluPrinter.print("  signature" + "\n");

					mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();
			}

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			mBluPrinter.print(mReceiptDataModel.cardHolderName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("I AGREE TO PAY AS PER CARD ISSUER AGREEMENT" + "\n");
			mBluPrinter.print("ACCORDING TO THE CARD ISSUER AGREEMENT" + "\n");

			printLine();

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			if(isCustomerCopy){
				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  CUSTOMER COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** CUSTOMER COPY ***" + "\n");
				}
			}
			else {
				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  MERCHANT COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** MERCHANT COPY ***" + "\n");
				}
			}

			mBluPrinter.print("Version No: " + mReceiptDataModel.appVersion + "\n");

			String bankName = getSettlementBank(mReceiptDataModel.bankName);

			if(bankName != null)
			{
				mBluPrinter.print("Settlement Bank : " + bankName);
			}

			mBluPrinter.setLineFeed(6);

			try {
				Thread.sleep(7000);
			}  catch(Exception e) {
				e.printStackTrace();
			}

			return true;

		} catch(Exception e) {
			e.printStackTrace();
		}

		return false;

	}

	private boolean generateNBFCEMiSaleReceipt()
	{
		try {

			if (mReceiptDataModel.emiPerMonthAmount.length() <= 0) {
				mReceiptDataModel.emiPerMonthAmount = "0.00";
			}

			if (mReceiptDataModel.total_Pay_Amount.length() <= 0) {
				mReceiptDataModel.total_Pay_Amount = "0.00";
			}

			double emiPerMonthAmountDouble = Double.parseDouble(mReceiptDataModel.emiPerMonthAmount.toString());
			double totalPayAmountDouble = Double.parseDouble(mReceiptDataModel.total_Pay_Amount.toString());

			mBluPrinter.POS_LineSpacing((byte) 2);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			Bitmap icon = getHeaderLogo();

			if(icon != null) {

				int targetWidth = 150;
				int targetHeight = icon.getHeight();

				Bitmap scaledBitmap = Bitmap.createScaledBitmap(icon, targetWidth, targetHeight, false);
				mBluPrinter.printImage(scaledBitmap, 0);

			}

			mBluPrinter.setLineFeed(1);
			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			mBluPrinter.print(mReceiptDataModel.merchantName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			String splitStr = "<br />";

			if (mReceiptDataModel.merchantAdd.contains("\n")) {
				splitStr = "\n";
			}

			for (String add : mReceiptDataModel.merchantAdd.split(splitStr)) {

				if (ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, "address  " + add, true, true);

				mBluPrinter.print(add + "\n");

			}


			String mDate = mReceiptDataModel.dateTime.split(" ")[0];
			String mTime = mReceiptDataModel.dateTime.split(" ")[1];

			try {

				String mAmPm = mReceiptDataModel.dateTime.split(" ")[2];

				if (mAmPm.length() > 0) {
					mTime = mTime + " " + mAmPm;
				}

			} catch (Exception e) {
				// TODO: handle exception
			}

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			printMediumFontLine("Date: " + mDate + "\n");

			printMediumFontLine("Time: " + mTime + "\n");

			mBluPrinter.print("MID: " + mReceiptDataModel.mId + "\n");

			mBluPrinter.print("TID: " + mReceiptDataModel.tId + "\n");

			mBluPrinter.print("Batch No.: " + mReceiptDataModel.batchNo + "\n");

			mBluPrinter.print("EMI" + "\n");

			mBluPrinter.print("RRN: " + mReceiptDataModel.rrNo + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			if (mReceiptDataModel.MfrName != null && mReceiptDataModel.MfrName.length() > 0)
			{
				mBluPrinter.print("Product Details" + "\n");

				mBluPrinter.print("Brand Name: " + mReceiptDataModel.MfrName + "\n");

				mBluPrinter.print("Prod Category: " + mReceiptDataModel.ProdCategory + "\n");

				mBluPrinter.print("Prod Name: " + mReceiptDataModel.ProdDesc + "\n");

				mBluPrinter.print("Prod Serial: " + mReceiptDataModel.ProdSerial + "\n");

				mBluPrinter.print("Offer Name: " + mReceiptDataModel.OfferTitle + "\n");

				mBluPrinter.print("Mobile: " + mReceiptDataModel.Mobile + "\n");

				double instDiscPer = 0;
				double baseAmount = 0;

				if(mReceiptDataModel.instDiscPer != null && mReceiptDataModel.instDiscPer.trim().length() > 0)
				{
					instDiscPer = Double.parseDouble(Constants.removeChar(mReceiptDataModel.instDiscPer,','));

					if(instDiscPer > 0)
					{
						String strInstDiscPer = Constants.getAmountWithComma(String.format("%.2f", instDiscPer));

						baseAmount = Double.parseDouble(Constants.removeChar(mReceiptDataModel.baseAmount, ','));
						String strbaseAmount = Constants.getAmountWithComma(String.format("%.2f", (baseAmount + instDiscPer)));

						mBluPrinter.print("Prod Amt: " + mReceiptDataModel.currency + " " + strbaseAmount + "\n");

						mBluPrinter.print("Inst Discount: " + mReceiptDataModel.currency + " " + strInstDiscPer + "\n");

					}
				}

				String loanAmt = "", merchPaybackAmt = "", emiPerMonthAmt = "",totalAmt = "";

				try{
					loanAmt = Constants.getAmountWithComma(String.format("%.2f", (Double.parseDouble(mReceiptDataModel.loanAmount))));
				}
				catch (NumberFormatException e){
					e.printStackTrace();
				}
				try{
					merchPaybackAmt = Constants.getAmountWithComma(String.format("%.2f", Double.parseDouble(mReceiptDataModel.discount)));
				}
				catch (NumberFormatException e){
					e.printStackTrace();
				}
				try{
					emiPerMonthAmt = Constants.getAmountWithComma(String.format("%.2f", Double.parseDouble(mReceiptDataModel.emiPerMonthAmount)));
				}
				catch (NumberFormatException e){
					e.printStackTrace();
				}
				try{
					totalAmt = Constants.getAmountWithComma(String.format("%.2f", Double.parseDouble(mReceiptDataModel.total_Pay_Amount)));
				}
				catch (NumberFormatException e){
					e.printStackTrace();
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);


				mBluPrinter.print("Tenure: " + mReceiptDataModel.noOfEmi + " Months" + "\n");

				mBluPrinter.print("Trx Amt: " + mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount + "\n");

				mBluPrinter.print("Merch Payback: " + mReceiptDataModel.emiPer + "%" + "\n");

				mBluPrinter.print("Merch Payback Amt: " + merchPaybackAmt + "\n");

				mBluPrinter.print("Loan amount: " + loanAmt + "\n");

				mBluPrinter.print("Easy Emi Fin Charge: " + mReceiptDataModel.emiRate  + "%" + "\n");

				mBluPrinter.print("EMI Amt: " + mReceiptDataModel.currency + " " + emiPerMonthAmt + "\n");

				mBluPrinter.print("Total Amount:  "+ (mReceiptDataModel.currency + " " + totalAmt) + "\n");

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("CUSTOMER CONSENT FOR EMI" + "\n");

				if(ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, " TermsConditionse " + mReceiptDataModel.TermsConditionse , true, true);
				if(ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, " CBTermsConditionse " + mReceiptDataModel.CBTermsConditionse , true, true);

				if(mReceiptDataModel.OfferTitle.equalsIgnoreCase("cashback"))
				{
					if(mReceiptDataModel.CBTermsConditionse != null && mReceiptDataModel.CBTermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.CBTermsConditionse + "\n");

					}
				}
				else if(mReceiptDataModel.OfferTitle.equalsIgnoreCase("zero emi"))
				{
					if(mReceiptDataModel.TermsConditionse != null && mReceiptDataModel.TermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.TermsConditionse + "\n");

					}
				}
				else
				{
					if(mReceiptDataModel.TermsConditionse != null && mReceiptDataModel.TermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.TermsConditionse + "\n");

					}

					if(mReceiptDataModel.CBTermsConditionse != null && mReceiptDataModel.CBTermsConditionse.length() >0) {
						mBluPrinter.print(mReceiptDataModel.CBTermsConditionse + "\n");

					}
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

			}
			else {

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("Tenure: " + mReceiptDataModel.noOfEmi + " Months" + "\n");

				mBluPrinter.print("Base Amt: " + "\n");

				mBluPrinter.print(mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount + "\n");

				mBluPrinter.print("Applicable Intr. Rate (P.A.) : " + mReceiptDataModel.interestRate + "%" + "\n");

				mBluPrinter.print("EMI Amt: " + "\n");

				mBluPrinter.print(mReceiptDataModel.currency + " " + df.format(emiPerMonthAmountDouble).toString() + "\n");

				mBluPrinter.print("Total Amt Payable: " + "\n");

				mBluPrinter.print(mReceiptDataModel.currency + " " + df.format(totalPayAmountDouble).toString() + "\n");

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("CUSTOMER CONSENT FOR EMI" + "\n");

				if(mReceiptDataModel.TermsConditionse != null && mReceiptDataModel.TermsConditionse.length() >0)
				{
					String splitStrr = "<br />";

					if (mReceiptDataModel.TermsConditionse.contains("\n")) {
						splitStrr = "\n";
					}

					for (String add : mReceiptDataModel.TermsConditionse.split(splitStrr)) {

						if (ApplicationData.IS_DEBUGGING_ON)
							Logs.v(ApplicationData.packName, "address  " + add, true, true);


						mBluPrinter.print(add + "\n");
					}
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				mBluPrinter.print("Base Amt: " + mReceiptDataModel.currency + " " + mReceiptDataModel.baseAmount + "\n");

				printLargeFontLine("Total Amt Payable (Incl. Interest): " + mReceiptDataModel.currency + " " + df.format(totalPayAmountDouble) + "\n");

			}

			if (mReceiptDataModel.isPinVarifed.equalsIgnoreCase("true")) {


				//mBluPrinter.POS_Set_Char_Mode(FONT_BOLD);

				mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);
				mBluPrinter.print("Pin Verified Ok" + "\n");

				mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			}else{

				if (isPrintSignatureRequired) {

					mBluPrinter.printImage(croppedBitmap, 0);

				}else{

					mBluPrinter.print("               " + "\n");

					mBluPrinter.POS_Set_Text_alingment(ALIGN_RIGHT);

					mBluPrinter.print("  signature" + "\n");

					mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);
				}

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

				printLine();

				mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			}

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			mBluPrinter.print(mReceiptDataModel.cardHolderName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			printLine();
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("I AGREE TO PAY THE ABOVE TOTAL AMOUNT" + "\n");
			mBluPrinter.print("I AGREE TO PAY AS PER CARD ISSUER AGREEMENT" + "\n");

			printLine();
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			if(isCustomerCopy){
				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  CUSTOMER COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** CUSTOMER COPY ***" + "\n");
				}
			}
			else {
				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  MERCHANT COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** MERCHANT COPY ***" + "\n");
				}
			}

			mBluPrinter.setLineFeed(6);

			return true;

		}catch (Exception e){
			e.printStackTrace();
		}

		return false;
	}

	private boolean genCashSaleReceipt()
	{

		if (ApplicationData.IS_DEBUGGING_ON)
			Logs.v(ApplicationData.packName, " genCashSaleReceipt " , true, true);

		try {

			mBluPrinter.POS_LineSpacing((byte) 2);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			Bitmap icon = getHeaderLogo();

			if(icon != null)
			{
				int targetWidth = 150;
				int targetHeight = icon.getHeight();

				Bitmap scaledBitmap = Bitmap.createScaledBitmap(icon, targetWidth, targetHeight, false);
				mBluPrinter.printImage(scaledBitmap, 0);
			}

			String address = "";

			try {

				address = (Html.fromHtml(mReceiptDataModel.merchantAdd)).toString();
			} catch (Exception e) {
				// TODO: handle exception
				address = mReceiptDataModel.merchantAdd;
			}


			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			mBluPrinter.print(mReceiptDataModel.merchantName + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			String splitStr = "<br />";

			if (mReceiptDataModel.merchantAdd.contains("\n")) {
				splitStr = "\n";
			}

			for (String add : mReceiptDataModel.merchantAdd.split(splitStr)) {

				if (ApplicationData.IS_DEBUGGING_ON)
					Logs.v(ApplicationData.packName, "address  " + add, true, true);

				mBluPrinter.print(add + "\n");

			}

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			printLine();

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);

			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

			mBluPrinter.print(mReceiptDataModel.dateTime + "\n");
			mBluPrinter.print("INVOICE NO:" +mReceiptDataModel.invoiceNo + "\n");
			mBluPrinter.print("REF. NO.:"+mReceiptDataModel.voucherNo + "\n");

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);
			mBluPrinter.print("\n" + mReceiptDataModel.trxType + "\n");

			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);
			mBluPrinter.print("TOTAL AMOUNT:   " + "\n");

			printLargeFontLine(mReceiptDataModel.currency + " " +mReceiptDataModel.totalAmount + "\n");

			if (isPrintSignatureRequired) {

				mBluPrinter.printImage(croppedBitmap, 0);

			}else{

				mBluPrinter.print("               " + "\n");

				mBluPrinter.print("  signature" + "\n");

			}

			printLine();

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("I AGREE TO PAY THE ABOVE TOTAL AMOUNT" + "\n");

			mBluPrinter.print("ACCORDING TO THE CARD ISSUER AGREEMENT" + "\n");

			printLine();

			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			if(isCustomerCopy){
				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  CUSTOMER COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** CUSTOMER COPY ***" + "\n");
				}
			}
			else {
				if(isDuplicatePrint) {
					mBluPrinter.print("*** DUPLICATE  MERCHANT COPY ***" + "\n");
				}else {
					mBluPrinter.print("*** MERCHANT COPY ***" + "\n");
				}
			}

			mBluPrinter.POS_Set_Text_alingment(ALIGN_RIGHT);

			mBluPrinter.print("Version No :"+ mReceiptDataModel.appVersion + "\n");

			mBluPrinter.setLineFeed(6);

			try {
				Thread.sleep(5000);
			}  catch(Exception e) {
				e.printStackTrace();
			}

			return true;

		} catch(Exception e) {
			e.printStackTrace();
		}

		return false;
	}

	private String removeampm(String aDate)
	{
		//as the date issue from the service level we are removing the am/pm from the date
		//we are getting with 24 form include am/pm which createing the error while converting.

		aDate.replaceAll("am", "");
		aDate.replaceAll("AM", "");
		aDate.replaceAll("pm", "");
		aDate.replaceAll("PM", "");

		return aDate.trim();

	}


	private void printLine(){

		try {

			mBluPrinter.POS_Set_Char_Mode(FONT_MEDIUM);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("-----------------------------------------" + "\n");

			mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_LEFT);

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private Bitmap getHeaderLogo(){

		Bitmap icon = BitmapFactory.decodeResource(context.getResources(),	R.drawable.mswipe2);
		if(icon != null) {
			return icon;
		}

		return null;

	}

	private String getSettlementBank(String bankName){

		if (bankName != null && bankName.length() > 0) {

			return bankName;
		}

		return  null;

	}

	private void printLargeFontLine(String line) throws IOException {
		mBluPrinter.POS_Set_Char_Mode(FONT_LARGE);
		mBluPrinter.print(line);
		mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
	}

	private void printMediumFontLine(String s) throws IOException {
		mBluPrinter.POS_Set_Char_Mode(FONT_MEDIUM);
		mBluPrinter.print(s);
		mBluPrinter.POS_Set_Char_Mode(FONT_SMALL);
	}


	public void print()
	{

		try {

			mBluPrinter.POS_Set_Char_Mode(FONT_MEDIUM);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print("-----------------------------------------" + "\n");
			mBluPrinter.print("sample text" + "\n");
			mBluPrinter.print("-----------------------------------------" + "\n");


			mBluPrinter.setLineFeed(4);


		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	public void print(String text)
	{

		try {

			mBluPrinter.POS_Set_Char_Mode(FONT_MEDIUM);
			mBluPrinter.POS_Set_Text_alingment(ALIGN_CENTER);

			mBluPrinter.print(text + "\n");


			mBluPrinter.setLineFeed(4);

		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	public void printBitmap(Bitmap icon)
	{

		try {

			if(icon != null) {

				int targetWidth = 150;
				int targetHeight = icon.getHeight();

				Bitmap scaledBitmap = Bitmap.createScaledBitmap(icon, targetWidth, targetHeight, false);

				mBluPrinter.printImage(scaledBitmap, 0);

			}

			mBluPrinter.setLineFeed(4);

		} catch(Exception e) {
			e.printStackTrace();
		}
	}
}