package com.mswipetech.wisepos.demo.sdk.view.Bluetoothfeatures;

import android.app.Activity;
import android.app.Dialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.bbpos.simplyprint.SimplyPrintController;
import com.mswipetech.wisepad.sdk.Print;
import com.mswipetech.wisepos.demo.sdk.Manager.services.MswipePrinterListner;
import com.mswipetech.wisepos.demo.sdk.Manager.services.MswipePrinterService;
import com.mswipetech.wisepos.demo.sdk.R;
import com.mswipetech.wisepos.demo.sdk.customviews.ListAdapter;
import com.mswipetech.wisepos.demo.sdk.data.ApplicationData;
import com.mswipetech.wisepos.demo.sdk.util.Constants;
import com.mswipetech.wisepos.demo.sdk.util.Logs;
import com.mswipetech.wisepos.demo.sdk.util.ReceiptUtilityBlu;

import java.util.ArrayList;
import java.util.Hashtable;

public class PrinterBluetoothDemo extends Activity {

    private Button mBTNPrintText = null;
    private Button mBTNPrintImage = null;
    private Button mBTNPrintEnterText = null;
    private EditText mETPrintEnterText = null;

    MswipePrinterService mPrinterConnectionService;
    ReceiptUtilityBlu receiptUtility;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_printer_demo);

        mBTNPrintText = (Button) findViewById(R.id.printdemo_BTN_printtext);
        mBTNPrintImage = (Button) findViewById(R.id.printdemo_BTN_printimage);
        mBTNPrintEnterText = (Button) findViewById(R.id.printdemo_BTN_printentered);
        mETPrintEnterText = (EditText) findViewById(R.id.printdemo_TXT_entertext);

        mBTNPrintText.setOnClickListener(view -> {

            if(isBound){
                if (mPrinterConnectionService.printerState == MswipePrinterListner.PRINTER_STATE.CONNECTED)
                {
                    receiptUtility.print();
                }
                else
                {
                    mPrinterConnectionService.connectToDevice();
                }
            }
            else{
                doBindService();
            }

        });

        mBTNPrintImage.setOnClickListener(view -> {

            if(isBound) {
                if (mPrinterConnectionService.printerState == MswipePrinterListner.PRINTER_STATE.CONNECTED)
                {
                    Print.init(PrinterBluetoothDemo.this);
                    Bitmap icon = BitmapFactory.decodeResource(getApplicationContext().getResources(), R.drawable.mswipe);
                    receiptUtility.printBitmap(icon);
                }
                else
                {
                    mPrinterConnectionService.connectToDevice();
                }
            }
            else{
                doBindService();
            }
        });

        mBTNPrintEnterText.setOnClickListener(view -> {

            if(isBound) {
                if (mPrinterConnectionService.printerState == MswipePrinterListner.PRINTER_STATE.CONNECTED) {
                    String text = mETPrintEnterText.getText().toString();
                    if (text != null && text.length() > 0) {
                        receiptUtility.print(text);
                    }
                } else {
                    mPrinterConnectionService.connectToDevice();
                }
            }
            else{
                doBindService();
            }
        });

        doBindService();

    }

    boolean isBound = false;


    void doBindService() {

        bindService(new Intent(this, MswipePrinterService.class), mConnection, Context.BIND_AUTO_CREATE);
        isBound = true;

    }

    @Override
    protected void onStop() {
        super.onStop();
        doUnbindService();
    }

    void doUnbindService() {

        if (isBound) {
            // If we have received the service, and hence registered with it,
            // then now is the time to unregister.
            // Detach our existing connection.
            unbindService(mConnection);
            isBound = false;
        }
    }

    private ServiceConnection mConnection = new ServiceConnection() {

        public void onServiceConnected(ComponentName className, IBinder service) {

            try {

                MswipePrinterService.LocalBinder localBinder = (MswipePrinterService.LocalBinder) service;
                mPrinterConnectionService =  localBinder.getService();
                mPrinterConnectionService.setPrinterListner(new PrinterListner());

            } catch (Exception e) {
                // In this case the service has crashed before we could even do
                // anything with it

                Logs.e(ApplicationData.packName,e,true,true);
            }

        }

        public void onServiceDisconnected(ComponentName className) {
            // This is called when the connection with the service has been
            // unexpectedly topbar_img_host_inactive - process crashed.
            mPrinterConnectionService = null;
        }
    };


    class PrinterListner extends MswipePrinterListner {


        @Override
        public void onRegisterd() {
            // TODO Auto-generated method stub
            super.onRegisterd();

        }

        @Override
        public void onUnRegisterd() {
            // TODO Auto-generated method stub
            super.onUnRegisterd();
        }


        @Override
        public void onPrinterStateChanged(PRINTER_STATE state) {
            // TODO Auto-generated method stub
            super.onPrinterStateChanged(state);

            if (ApplicationData.IS_DEBUGGING_ON)
                Logs.v(ApplicationData.packName, "printer state "+state, true, true);

            if (state == PRINTER_STATE.WAITINGFORCONNECTION) {


                final Dialog dialog = Constants.showDialog(PrinterBluetoothDemo.this, "",
                        getString(R.string.connecting_to_printer_if_its_taking_longer_than_usual_please_restart_the_printer_and_try_reconnecting), Constants.CUSTOM_DLG_TYPE.CUSTOM_DLG_TYPE_RETURN_DLG_INFO);

                Button yes = (Button) dialog.findViewById(R.id.customdlg_BTN_yes);
                yes.setOnClickListener(new View.OnClickListener() {

                    public void onClick(View v) {
                        dialog.dismiss();

                    }
                });

                dialog.show();

            }else if (state == PRINTER_STATE.CONNECTED) {

                if(mPrinterConnectionService.bluetoothBluedevice){
                    receiptUtility = new ReceiptUtilityBlu(PrinterBluetoothDemo.this, mPrinterConnectionService.mBLuPrinter);
                }

            }else if (state == PRINTER_STATE.DISCONNECTED) {


            }else if (state == PRINTER_STATE.PRINTING) {


            }
        }


        @Override
        public void onPrinterConnectionError(PRINTER_CONNECTION_ERROR state, ArrayList<BluetoothDevice> devices) {
            // TODO Auto-generated method stub
            super.onPrinterConnectionError(state, devices);

            String msg = "";

            if(state == PRINTER_CONNECTION_ERROR.NO_PAIRED_DEVICE_FOUND){

                msg = getString(R.string.no_paired_printer_found_please_pair_the_printer_from_your_phones_bluetooth_settings_and_try_again);

            }else if(state == PRINTER_CONNECTION_ERROR.MULTIPLE_PAIRED_DEVICE){

                pairedDevicesFound = devices;

                TaskShowMultiplePairedDevices pairedtask = new TaskShowMultiplePairedDevices();
                pairedtask.execute();

                return;
            }else if (state == PRINTER_CONNECTION_ERROR.BLUETOOTH_OFF) {

                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableBtIntent, 1000);

            }else if (state == PRINTER_CONNECTION_ERROR.UNKNOWN) {

                msg = getString(R.string.unable_to_connect_to_the_bluetooth_printer);

            }else if (state == PRINTER_CONNECTION_ERROR.CONNECTING){
                msg = getString(R.string.connecting_to_printer_if_its_taking_longer_than_usual_please_restart_the_printer_and_try_reconnecting);
            }else if (state == PRINTER_CONNECTION_ERROR.WISEPAD_SWITCHED_OFF){
                final Dialog dialog = Constants.showDialog(PrinterBluetoothDemo.this, "",
                        getString(R.string.printer_not_connected_please_make_sure_that_the_printer_is_switched_on), Constants.CUSTOM_DLG_TYPE.CUSTOM_DLG_TYPE_RETURN_DLG_INFO,
                        getResources().getString(R.string.connect),
                        getResources().getString(R.string.close));

                Button yes = (Button) dialog.findViewById(R.id.customdlg_BTN_yes);
                ((Button) dialog.findViewById(R.id.customdlg_BTN_yes)).setText(getResources().getString(R.string.ok));
                yes.setOnClickListener(new View.OnClickListener() {

                    public void onClick(View v) {
                        dialog.dismiss();
                        mPrinterConnectionService.reconnectToDevice();
                    }
                });


                dialog.show();

                return;

            }else if(state == PRINTER_CONNECTION_ERROR.PRINTING_IN_PROGRESS){



                new Handler(Looper.getMainLooper()).post(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(PrinterBluetoothDemo.this, getString(R.string.printing_in_process), Toast.LENGTH_SHORT).show();
                    }
                });

            }

            if (ApplicationData.IS_DEBUGGING_ON)
                Logs.v(ApplicationData.packName,"makeConnnection msg "+ msg,true, true);

            if (msg.length()>0) {
                showPrinterDialog(msg);
            }
        }



        @Override
        public void onReturnDeviceInfo(Hashtable<String, String> deviceInfoTable) {

        }

        @Override
        public void onReturnPrinterResult(SimplyPrintController.PrinterResult printerResult) {

            String msg = "";

            if(printerResult == SimplyPrintController.PrinterResult.SUCCESS) {
                msg =  getString(R.string.printer_command_success);
            } else if(printerResult == SimplyPrintController.PrinterResult.NO_PAPER) {
                msg =  getString(R.string.no_paper);
            } else if(printerResult == SimplyPrintController.PrinterResult.WRONG_CMD) {
                msg =  getString(R.string.wrong_printer_cmd);
            } else if(printerResult == SimplyPrintController.PrinterResult.OVERHEAT) {
                msg =  getString(R.string.printer_overheat);
            }

            if (ApplicationData.IS_DEBUGGING_ON)
                Logs.v(ApplicationData.packName, "onReturnPrinterResult PrinterResult "+ msg, true, true);

            showPrinterDialog(msg);

        }

        @Override
        public void onPrinterOperationEnd() {


        }

        @Override
        public void onBatteryLow(SimplyPrintController.BatteryStatus batteryStatus) {

            String msg = "";

            if(batteryStatus == SimplyPrintController.BatteryStatus.LOW) {
                msg =  getString(R.string.printer_battery_low);
            } else if(batteryStatus == SimplyPrintController.BatteryStatus.CRITICALLY_LOW) {
                msg =  getString(R.string.printer_battery_critically_low);
            }

            try{

                Constants.showDialog(PrinterBluetoothDemo.this, "", msg);

            }catch(Exception e){
                e.printStackTrace();
            }

        }

        @Override
        public void onError(SimplyPrintController.Error errorState) {

            String msg = "";


            if(errorState == SimplyPrintController.Error.UNKNOWN) {
                msg =  getString(R.string.printer_unknown_error);
            } else if(errorState == SimplyPrintController.Error.CMD_NOT_AVAILABLE) {
                msg =  getString(R.string.printer_command_not_available);
            } else if(errorState == SimplyPrintController.Error.TIMEOUT) {
                msg =  getString(R.string.printer_device_no_response);
            } else if(errorState == SimplyPrintController.Error.DEVICE_BUSY) {
                msg =  getString(R.string.printer_device_busy);
            } else if(errorState == SimplyPrintController.Error.INPUT_OUT_OF_RANGE) {
                msg =  getString(R.string.printer_out_of_range);
            } else if(errorState == SimplyPrintController.Error.INPUT_INVALID) {
                msg =  getString(R.string.printer_input_invalid);
            } else if(errorState == SimplyPrintController.Error.CRC_ERROR) {
                msg =  getString(R.string.printer_crc_error);
            } else if(errorState == SimplyPrintController.Error.FAIL_TO_START_BTV2) {
                msg =  getString(R.string.printer_fail_to_start_bluetooth);
            } else if(errorState == SimplyPrintController.Error.COMM_LINK_UNINITIALIZED) {
                msg =  getString(R.string.printer_comm_link_uninitialized);
            } else if(errorState == SimplyPrintController.Error.BTV2_ALREADY_STARTED) {
                msg =  getString(R.string.printer_bluetooth_already_started);
            }


            if (ApplicationData.IS_DEBUGGING_ON)
                Logs.v(ApplicationData.packName, "onError  the error state is " + errorState, true, true);

            showPrinterDialog(msg);

        }

    }

    public void printBluReceipt() {

        receiptUtility.print();
    }

    private void showPrinterDialog(String msg)
    {
        try
        {

            Dialog mDialogPrinter =  new Dialog(PrinterBluetoothDemo.this);
            mDialogPrinter.setContentView(R.layout.customdlg);
            mDialogPrinter.setCanceledOnTouchOutside(false);

            mDialogPrinter.setCancelable(true);

            ApplicationData applicationData = ApplicationData.getApplicationDataSharedInstance();

            // set the title
            TextView txttitle = (TextView) mDialogPrinter.findViewById(R.id.customdlg_TXT_title);
            txttitle.setText("");

            // to set the message
            TextView txtMessage = (TextView) mDialogPrinter.findViewById(R.id.customdlg_TXT_Info);
            txtMessage.setText(msg);

            Button yes = (Button) mDialogPrinter.findViewById(R.id.customdlg_BTN_yes);
				/*yes.setText("ok");
				yes.setTypeface(applicationData.font);*/

            Button no = (Button) mDialogPrinter.findViewById(R.id.customdlg_BTN_no);

            ((Button) mDialogPrinter.findViewById(R.id.customdlg_BTN_yes)).setText(getResources().getString(R.string.ok));
            ((Button) mDialogPrinter.findViewById(R.id.customdlg_BTN_no)).setText(getResources().getString(R.string.cancel));

				/*no.setText("clear");
				no.setTypeface(applicationData.font);*/
            no.setVisibility(View.GONE);
            yes.setOnClickListener(new View.OnClickListener() {

                public void onClick(View v) {
                    mDialogPrinter.dismiss();

                }
            });

            mDialogPrinter.getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            mDialogPrinter.show();

        }catch(Exception e){
            e.printStackTrace();
        }
    }

    ArrayList<BluetoothDevice> pairedDevicesFound = new ArrayList<BluetoothDevice>();

    private class TaskShowMultiplePairedDevices extends AsyncTask<String, Void, String> {
        @Override
        protected String doInBackground(String... urls) {
            return "";
        }

        @Override
        protected void onPostExecute(String result) {

            final Dialog dlgPairedDevices = Constants.showAppCustomDialog(PrinterBluetoothDemo.this, getResources().getString(R.string.mswisepadview_select_wisepad));
            dlgPairedDevices.setCancelable(true);

            final ArrayList<String> deviceNameList = new ArrayList<String>();

            for (int i = 0; i < pairedDevicesFound.size(); ++i)
            {
                deviceNameList.add(pairedDevicesFound.get(i).getName());
            }


            final ListView appListView = (ListView) dlgPairedDevices.findViewById(R.id.customapplicationdlg_LST_applications);

            appListView.setAdapter(new ListAdapter(PrinterBluetoothDemo.this, deviceNameList));
            appListView.setOnItemClickListener(new AdapterView.OnItemClickListener()
            {

                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {


                    if (mPrinterConnectionService != null)
                        mPrinterConnectionService.connectToWisePad(pairedDevicesFound.get(position));

                    dlgPairedDevices.dismiss();
                }

            });

            dlgPairedDevices.findViewById(R.id.customapplicationdlg_BTN_cancel).setOnClickListener(new View.OnClickListener() {

                @Override
                public void onClick(View v) {

                    dlgPairedDevices.dismiss();
                }
            });

            dlgPairedDevices.show();


        }
    }
}
