package com.mswipetech.wisepos.demo.sdk.view;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.TextView;

import com.mswipetech.wisepad.sdk.MSWisepadController;
import com.mswipetech.wisepad.sdk.data.MSDataStore;
import com.mswipetech.wisepad.sdk.data.ResendReceiptResponsedata;
import com.mswipetech.wisepad.sdk.listeners.MSWisepadControllerResponseListener;
import com.mswipetech.wisepos.demo.sdk.R;
import com.mswipetech.wisepos.demo.sdk.customviews.CustomProgressDialog;
import com.mswipetech.wisepos.demo.sdk.data.AppSharedPrefrences;
import com.mswipetech.wisepos.demo.sdk.util.Constants;
import com.mswipetech.wisepos.demo.sdk.view.cardpaymentactivity.LastTransactionActivity;
import com.mswipetech.wisepos.demo.sdk.view.login.LoginView;

public class ResendReceiptActivity extends Activity {

    private TextView mTxtMobileNo = null;
    private TextView mTxtEmail = null;

    private String mStrAmount = "";
    private String mStrLastFourDigits = "";
    private String mStrStanNo = "";

    CustomProgressDialog mProgressActivity = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.activity_resend_receipt_on_phone);

        mTxtMobileNo = (TextView) findViewById(R.id.resend_mobile_et);
        mTxtEmail = (TextView) findViewById(R.id.resend_email_et);

        mStrAmount = getIntent().getStringExtra("amount");
        mStrLastFourDigits = getIntent().getStringExtra("lastfourdigits");
        mStrStanNo = getIntent().getStringExtra("stanno");

        ((TextView) findViewById(R.id.topbar_LBL_heading)).setText("Resend Receipt");

        Button btnLogin = (Button) findViewById(R.id.btn_send_receipt);
        btnLogin.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {

                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(ResendReceiptActivity.this.getCurrentFocus().getWindowToken(),
                        InputMethodManager.HIDE_NOT_ALWAYS);

                callResendReceipt();

            }
        });
    }



    private void callResendReceipt(){

        if (( mTxtMobileNo.getText().toString().length() < 10) ||  mTxtMobileNo.getText().toString().length() == 0) {
            Constants.showDialog(ResendReceiptActivity.this, "Resend Receipt", "Please enter phone no or email id");
            return;
        }

        final Dialog dialog = Constants.showDialog(ResendReceiptActivity.this, getString(R.string.lasttransaction_view_status),
                getString(R.string.lasttransactionactivity_would_you_like_to_resend_the_receipt_for_the_selected_transaction),
                Constants.CUSTOM_DLG_TYPE.CUSTOM_DLG_TYPE_RETURN_DLG_CONFIRMATION);

        Button yes = (Button) dialog.findViewById(R.id.customdlg_BTN_yes);
        yes.setOnClickListener(new View.OnClickListener() {

            public void onClick(View v) {
                dialog.dismiss();
                resendReceipt();

            }
        });

        Button no = (Button) dialog.findViewById(R.id.customdlg_BTN_no);
        no.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                dialog.dismiss();

            }
        });

        dialog.show();

    }



    private void resendReceipt(){

        try {

            final MSWisepadController wisepadController = MSWisepadController.
                    getSharedMSWisepadController(ResendReceiptActivity.this,
                            AppSharedPrefrences.getAppSharedPrefrencesInstace().getGatewayEnvironment(),
                            AppSharedPrefrences.getAppSharedPrefrencesInstace().getNetworkSource(),
                            null);

            MSWisepadController.setNetworkSource(AppSharedPrefrences.getAppSharedPrefrencesInstace().getNetworkSource());

            wisepadController.processResendReceipt(
                    AppSharedPrefrences.getAppSharedPrefrencesInstace().getReferenceId(),
                    AppSharedPrefrences.getAppSharedPrefrencesInstace().getSessionToken(),
                    mStrAmount,
                    mStrLastFourDigits,
                    mStrStanNo,
                    mTxtMobileNo.getText().toString(),
                    mTxtEmail.getText().toString(),
                    new MSWisepadControllerResendReceipResponseListenerObserver());

            mProgressActivity = new CustomProgressDialog(ResendReceiptActivity.this, getString(R.string.processing));
            mProgressActivity.show();

        }
        catch (Exception e)
        {
            Constants.showDialog(this,getResources().getString(R.string.lasttransaction_view_status),getResources().getString(R.string.lasttransactionactivity_unable_to_process_last_transaction));
            e.printStackTrace();
        }
    }

    /**
     * MSWisepadControllerResendReceipResponseListenerObserver
     * The mswipe overridden class  observer which listens to the response to the mswipe sdk function requests

     */

    class MSWisepadControllerResendReceipResponseListenerObserver implements MSWisepadControllerResponseListener
    {

        /**
         * onReponseData
         * The response data notified back to the call back function
         * @param
         * aMSDataStore
         * 		the generic mswipe data store, this instance refers to LastTransactionResponseData, so this
         * need be type cast back to LastTransactionResponseData to access the  response data
         * @return
         */
        public void onReponseData(MSDataStore aMSDataStore)
        {


            ResendReceiptResponsedata resendReceiptResponsedata = (ResendReceiptResponsedata) aMSDataStore;

            boolean responseStatus = resendReceiptResponsedata.getResponseStatus();
            mProgressActivity.dismiss();

            if (!responseStatus)
            {


                final Dialog dialog = Constants.showDialog(ResendReceiptActivity.this, getString(R.string.lasttransactionactivity_last_tx), resendReceiptResponsedata.getResponseFailureReason()
                        , Constants.CUSTOM_DLG_TYPE.CUSTOM_DLG_TYPE_RETURN_DLG_INFO);
                Button yes = (Button) dialog.findViewById(R.id.customdlg_BTN_yes);
                yes.setOnClickListener(new View.OnClickListener() {

                    public void onClick(View v) {
                        dialog.dismiss();
                        finish();
                    }
                });
                dialog.show();
            }
            else if (responseStatus)
            {

                final Dialog dialog = Constants.showDialog(ResendReceiptActivity.this, getString(R.string.lasttransactionactivity_last_tx), resendReceiptResponsedata.getResponseSuccessMessage()
                        , Constants.CUSTOM_DLG_TYPE.CUSTOM_DLG_TYPE_RETURN_DLG_INFO);
                Button yes = (Button) dialog.findViewById(R.id.customdlg_BTN_yes);
                yes.setOnClickListener(new View.OnClickListener() {

                    public void onClick(View v) {
                        dialog.dismiss();
                        finish();
                    }
                });
                dialog.show();
            }
        }
    }

}