package com.mswipetech.wisepos.demo.sdk.view.login;

import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.mswipetech.sdk.network.MSGatewayConnectionListener;
import com.mswipetech.wisepad.sdk.MSWisepadController;
import com.mswipetech.wisepad.sdk.data.LoginResponseData;
import com.mswipetech.wisepad.sdk.data.MSDataStore;
import com.mswipetech.wisepad.sdk.listeners.MSWisepadControllerResponseListener;
import com.mswipetech.wisepos.demo.sdk.R;
import com.mswipetech.wisepos.demo.sdk.customviews.CustomProgressDialog;
import com.mswipetech.wisepos.demo.sdk.data.AppSharedPrefrences;
import com.mswipetech.wisepos.demo.sdk.data.ApplicationData;
import com.mswipetech.wisepos.demo.sdk.util.Constants;
import com.mswipetech.wisepos.demo.sdk.util.Constants.CUSTOM_DLG_TYPE;
import com.mswipetech.wisepos.demo.sdk.util.Logs;
import com.mswipetech.wisepos.demo.sdk.view.BaseTitleActivity;

/**
 * LoginView 
 * Login activity will unable the merchant to login into the mswipe system
 */

public class LoginOTPView extends BaseTitleActivity
{
	//The mobile no of the merchant identified as a unique id
    private TextView mTxtMerchantId = null;
    private TextView mTxtMerchantOTP = null;
    private Button btnLogin  = null;
    private Button btnOTP  = null;

    
    //The progress of the application activity  
    private CustomProgressDialog mProgressActivity = null;
    
    
    //The network connection listener observes all the network notifications to the mswipe gateways
    private MSGatewayConncetionObserver mMSGatewayConncetionObserver = null;
    
    //The mswipe controller observes all the responses from the mswipe gateway
    private MSWisepadControllerResponseListenerObserver mMSWisepadControllerResponseListenerObserver = null;
    private MSWisepadControllerResponseListenerObserverOTP mMSWisepadControllerResponseListenerObserverOTP = null;

  //The mswipe controller instance used for calling up the api's
    private MSWisepadController mMSWisepadController = null;

    /*images to show the bluetooth and  websocket connection status i.e wheteher topbar_img_host_active or not*/
    public ImageView imgHostConnectionStatus;

    /*using these animation for connection status*/
    public Animation alphaAnim;

    // this time will handle the duplicate transactions, by ignoring the click action for 5 seconds time interval
    private long lastRequestTime = 0;
  
    
    @Override
    public void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.view_otp_login);

        mMSGatewayConncetionObserver = new MSGatewayConncetionObserver();
        mMSWisepadControllerResponseListenerObserver = new MSWisepadControllerResponseListenerObserver();
        mMSWisepadControllerResponseListenerObserverOTP = new MSWisepadControllerResponseListenerObserverOTP();

        mMSWisepadController = MSWisepadController.
                getSharedMSWisepadController(getApplicationContext(),
                        AppSharedPrefrences.getAppSharedPrefrencesInstace().getGatewayEnvironment(),
                        AppSharedPrefrences.getAppSharedPrefrencesInstace().getNetworkSource(),
                        mMSGatewayConncetionObserver);


        MSWisepadController.setNetworkSource(AppSharedPrefrences.getAppSharedPrefrencesInstace().getNetworkSource());
        
        initViews();
    }

    /**
     * Initializes the UI elements for the login screen
     * @param  
     * 
     * @return 
     * 
     */
    private void initViews() 
    {


        imgHostConnectionStatus = (ImageView) findViewById(R.id.topbar_IMG_position2);
        imgHostConnectionStatus.setVisibility(View.VISIBLE);

        alphaAnim = AnimationUtils.loadAnimation(this, R.anim.alpha_anim);

    	mTxtMerchantId = (TextView) findViewById(R.id.login_TXT_merchantid);
        mTxtMerchantOTP = (TextView) findViewById(R.id.login_TXT_merchantotp);
        mTxtMerchantOTP.setVisibility(View.GONE);
        
        ((TextView) findViewById(R.id.topbar_LBL_heading)).setText("Login with OTP");

        btnOTP = (Button) findViewById(R.id.login_BTN_signotp);
        btnLogin = (Button) findViewById(R.id.login_BTN_signin);
        btnLogin.setVisibility(View.GONE);


        btnOTP.setOnClickListener(new OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(LoginOTPView.this.getCurrentFocus().getWindowToken(),
                        InputMethodManager.HIDE_NOT_ALWAYS);

                sendAuthenticateOTP();
            }
        });

        btnLogin.setOnClickListener(new OnClickListener()
        {
            @Override
            public void onClick(View v) 
            {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(LoginOTPView.this.getCurrentFocus().getWindowToken(),
                        InputMethodManager.HIDE_NOT_ALWAYS);

                authenticateMerchantWithOTP();

            }
        });


    }

    @Override
    protected void onStop() {
        super.onStop();

        mMSWisepadController.stopMSGatewayConnection();
    }

    /**
     * Validates the login credential of the merchant
     * @param  
     * @return
     */
    public void sendAuthenticateOTP()
    {

        if (System.currentTimeMillis() - lastRequestTime >= 3000) {

            if (mTxtMerchantId.getText().toString().length() < 0) {
                Constants.showDialog(LoginOTPView.this, "LoginView", "Please enter valid userid");

            }
            else {

                try
                {

                    mMSWisepadController.sendAuthenticateOTP(
                            mTxtMerchantId.getText().toString(),
                            mMSWisepadControllerResponseListenerObserverOTP);

                } catch (Exception e) {
                    e.printStackTrace();
                }

                mProgressActivity = new CustomProgressDialog(LoginOTPView.this, "Processing...");
                mProgressActivity.show();

            }

        }
        lastRequestTime = System.currentTimeMillis();

    }

    private void authenticateMerchantWithOTP() {

        if (System.currentTimeMillis() - lastRequestTime >= 3000) {

            if(mTxtMerchantOTP.getText().toString().length() < 6){
                Constants.showDialog(LoginOTPView.this, "LoginView", "Please enter valid otp");
            }
            else {

                try
                {
                    mMSWisepadController.authenticateMerchantWithOTP(
                            mTxtMerchantId.getText().toString(),
                            mTxtMerchantOTP.getText().toString(),
                            mMSWisepadControllerResponseListenerObserver);

                } catch (Exception e) {
                    e.printStackTrace();
                }

                mProgressActivity = new CustomProgressDialog(LoginOTPView.this, "Logging in...");
                mProgressActivity.show();

            }

        }
        lastRequestTime = System.currentTimeMillis();


    }
    /**
     * MSWisepadControllerResponseListenerObserver
     * The mswipe overridden class  observer which listens to the responses from the mswipe sdk function
     */
    class MSWisepadControllerResponseListenerObserver implements MSWisepadControllerResponseListener
    {
    	
    	/**
         * onReponseData 
         * The response data notified back to the call back function
         * @param  
         * aMSDataStore
         * 		the generic mswipe data store, this instance refers to LoginResponseData, so this 
         * need be type cast back to LoginResponseData to access the login response data
         * @return 
         */
    	public void onReponseData(MSDataStore aMSDataStore)
    	{
    		if(mProgressActivity != null)
    		{
    			mProgressActivity.dismiss();
    			mProgressActivity = null;
    		}
    		
    		LoginResponseData loginResponseData = (LoginResponseData) aMSDataStore;
    		
    		boolean responseStatus = loginResponseData.getResponseStatus();
    		
    		if (!responseStatus) 
    		{
                
                Constants.showDialog(LoginOTPView.this, "LoginView", loginResponseData.getResponseFailureReason());

            } 
    		else if (responseStatus){

                String FirstName =  loginResponseData.getFirstName();                   
                
                //below details be used to call up the rest of the api, so by saving them to the device stores like,
    			//shared preferences or sqllite, if this details are saved the login to the mswipe gateway is not
                //required again, 
                
                
    			//this Session_Tokeniser does not change until the password for the user id is not changed or if the same user has
                //logged in again.
    			String Reference_Id =  loginResponseData.getReferenceId();                   
                String Session_Tokeniser =  loginResponseData.getSessionTokeniser();                   
                
                //save the referenceId and Session_Tokeniser for accessing the other api's, this are a kind of outh
                //authenticated permission for accessing the other services
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setReferenceId(Reference_Id);
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setSessionToken(Session_Tokeniser);
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setUserId(mTxtMerchantId.getText().toString());
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setMID(loginResponseData.getMID());

                String Currency_Code =  loginResponseData.getCurrency();                   
                boolean tipRequired =  loginResponseData.isTipsRequired();                   
                float convienencePercentage = loginResponseData.getConveniencePercentage();                  
                float serviceTax =  loginResponseData.getServiceTax();
                
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setCurrencyCode(Currency_Code+".");
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setTipEnabled(tipRequired);
                
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setConveniencePercentage(convienencePercentage);
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setServicePercentageOnConvenience(serviceTax);

                AppSharedPrefrences.getAppSharedPrefrencesInstace().setPinBypass(loginResponseData.isPinBypass());
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setReceiptEnabled(loginResponseData.isReceiptRequired());

                AppSharedPrefrences.getAppSharedPrefrencesInstace().setBQREnabled(loginResponseData.isBQREnabled());
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setBQRIdsEnabled(loginResponseData.isBQRIdsEnabled());
                //AppSharedPrefrences.getAppSharedPrefrencesInstace().setBQRIds(loginResponseData.getBQRIds());
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setCustCode(loginResponseData.getCustCode());
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setPayByLink(loginResponseData.isPayByLink());

                AppSharedPrefrences.getAppSharedPrefrencesInstace().setVirtualPaymentId(loginResponseData.getVirtualPaymentId());
                AppSharedPrefrences.getAppSharedPrefrencesInstace().setPayByLinkBank(loginResponseData.getPayByLinkBank());

                boolean IS_Password_Changed = loginResponseData.isFirstTimePasswordChanged();
                
                //if this is false, then its suggest that the password has not changed since the user has 
                //been created at-least once
                if (!IS_Password_Changed) 
                {
                    startActivity(new Intent(LoginOTPView.this, ChangePasswordView.class));
                    finish();
                    return;
                    
                }
                else{
                	
                    final Dialog dlg = Constants.showDialog(LoginOTPView.this, "LoginView", FirstName + " User authenticated.",
                    		CUSTOM_DLG_TYPE.CUSTOM_DLG_TYPE_RETURN_DLG_INFO);
                    Button btnOk = (Button) dlg.findViewById(R.id.customdlg_BTN_yes);
                    btnOk.setOnClickListener(new OnClickListener() {
						
						@Override
						public void onClick(View arg0) {
							// TODO Auto-generated method stub
							dlg.dismiss();
							
							finish();
							
						}
					});
                    dlg.show();
                                     	
                }
            }
            else{
                
            	Constants.showDialog(LoginOTPView.this, "LoginView", "Invalid response from Mswipe server, please contact support.");
            }
    	}
    }


    class MSWisepadControllerResponseListenerObserverOTP implements MSWisepadControllerResponseListener
    {

    	/**
         * onReponseData
         * The response data notified back to the call back function
         * @param
         * aMSDataStore
         * 		the generic mswipe data store, this instance refers to LoginResponseData, so this
         * need be type cast back to LoginResponseData to access the login response data
         * @return
         */
    	public void onReponseData(MSDataStore aMSDataStore)
    	{
    		if(mProgressActivity != null)
    		{
    			mProgressActivity.dismiss();
    			mProgressActivity = null;
    		}

    		LoginResponseData loginResponseData = (LoginResponseData) aMSDataStore;

    		boolean responseStatus = loginResponseData.getResponseStatus();

    		if (!responseStatus)
    		{
    		    Constants.showDialog(LoginOTPView.this, "LoginView", loginResponseData.getResponseFailureReason());
            }
    		else if (responseStatus){

    		    mTxtMerchantOTP.setVisibility(View.VISIBLE);
                btnLogin.setVisibility(View.VISIBLE);
                btnOTP.setVisibility(View.GONE);

                Constants.showDialog(LoginOTPView.this, "LoginView", loginResponseData.getResponseSuccessMessage());
            }
            else{

            	Constants.showDialog(LoginOTPView.this, "LoginView", "Invalid response from Mswipe server, please contact support.");
            }
    	}
    }



    class MSGatewayConncetionObserver implements MSGatewayConnectionListener {

        @Override
        public void Connected(String msg) {

            imgHostConnectionStatus.setAnimation(null);
            imgHostConnectionStatus.setImageResource(R.drawable.topbar_img_host_active);
        }

        @Override
        public void Connecting(String msg) {

            if (ApplicationData.IS_DEBUGGING_ON)
                Logs.v(ApplicationData.packName, " msg " + msg, true, true);

            imgHostConnectionStatus.startAnimation(alphaAnim);
            imgHostConnectionStatus.setImageResource(R.drawable.topbar_img_host_inactive);
        }

        @Override
        public void disConnect(String msg) {


            if (ApplicationData.IS_DEBUGGING_ON)
                Logs.v(ApplicationData.packName, " msg " + msg, true, true);

            imgHostConnectionStatus.setAnimation(null);
            imgHostConnectionStatus.setImageResource(R.drawable.topbar_img_host_inactive);
        }
    }
 }